'use strict';

angular.module('webdiary', ['webdiary.controllers', 'webdiary.services', 'webdiary.directives', 'webdiary.filters', 'ui.select', 'ngSanitize'
            , 'ngAnimate', 'ui.bootstrap', 'ui.router', 'ui.grid', 'ui.grid.selection', 'ng-context-menu', 'bsLoadingOverlay', 'dndLists' /*, 'angularjs-dropdown-multiselect' /*'ngAutocomplete'*/])

        .run(function ($rootScope, Configuration, $state, $log, bsLoadingOverlayService) {
            Configuration.initialize();
            $rootScope.$on('$stateChangeStart', function (event, toState, toParams) {

                var requireLogin = toState.data.requireLogin;
                var token = Configuration.getConfiguration().jsonwebtoken;
                //$log.info("token = " + token );

                if (requireLogin && (token === '' || token === undefined)) {
                    //$log.info('preventDefault');
                    event.preventDefault();
                    // get me a login modal!

                    $state.go('login');
                }
            });
            bsLoadingOverlayService.setGlobalConfig({
                delay: 500,
                templateUrl: 'templates/overlay/loading-overlay.html' // Template url for overlay element. If not specified - no overlay element is created.

            });

        }).config(function ($stateProvider, $urlRouterProvider, $httpProvider) {

            
          $stateProvider
            .state('login', {
                url: '/login',
                templateUrl: "templates/login/login.html",
                controller: "LoginCtrl",
                data: {
                    requireLogin: false
                }
            })
            .state('base', {
                abstract: true,
                template: "<ui-view/>",
                data: {
                    requireLogin: true // this property will apply to all children of 'app'
                }
            })
            .state('base.diary', {
                url: '/diary',
                templateUrl: 'templates/diary/diary.html',
                controller: 'DiaryCtrl'
            })
            .state('base.actionsscreen', {
                url: '/driveractions',
                templateUrl: 'templates/driveractions/actionScreen.html',
                controller: 'ActionScreenCtrl'
            })
            .state('base.warehousescreen', {
                url: '/warehouseactions',
                templateUrl: 'templates/warehouse/warehouseScreen.html',
                controller: 'warehouseScreenCtrl'
            });

    // if none of the above states are matched, use this as the fallback
    $urlRouterProvider.otherwise('/login');
    // this interceptor handle adding authorization header with jwt token 
    // and also re authentication after a 401
    $httpProvider.interceptors.push('AuthInterceptor');



}).config(function ($provide) {
    $provide.decorator("$q", function ($delegate) {
        //Helper method copied from q.js.
        var isPromiseLike = function (obj) {
            return obj && angular.isFunction(obj.then);
        }

        /*
         * @description Execute a collection of tasks serially.  A task is a function that returns a promise
         *
         * @param {Array.<Function>|Object.<Function>} tasks An array or hash of tasks.  A tasks is a function
         *   that returns a promise.  You can also provide a collection of objects with a success tasks, failure task, and/or notify function
         * @returns {Promise} Returns a single promise that will be resolved or rejected when the last task
         *   has been resolved or rejected.
         */
        function serial(tasks) {
            //Fake a "previous task" for our initial iteration
            var prevPromise;
            var error = new Error();
            angular.forEach(tasks, function (task, key) {
                var success = task.success || task;
                var fail = task.fail;
                var notify = task.notify;
                var nextPromise;

                //First task
                if (!prevPromise) {
                    nextPromise = success();
                    if (!isPromiseLike(nextPromise)) {
                        error.message = "Task " + key + " did not return a promise.";
                        throw error;
                    }
                } else {
                    //Wait until the previous promise has resolved or rejected to execute the next task
                    nextPromise = prevPromise.then(
                            /*success*/function (data) {
                                if (!success) {
                                    return data;
                                }
                                var ret = success(data);
                                if (!isPromiseLike(ret)) {
                                    error.message = "Task " + key + " did not return a promise.";
                                    throw error;
                                }
                                return ret;
                            },
                            /*failure*/function (reason) {
                                if (!fail) {
                                    return $delegate.reject(reason);
                                }
                                var ret = fail(reason);
                                if (!isPromiseLike(ret)) {
                                    error.message = "Fail for task " + key + " did not return a promise.";
                                    throw error;
                                }
                                return ret;
                            },
                            notify);
                }
                prevPromise = nextPromise;
            });

            return prevPromise || $delegate.when();
        }

        $delegate.serial = serial;
        return $delegate;
    });
});

var diaryControllers = angular.module('webdiary.controllers', []);
var diaryDirectives = angular.module('webdiary.directives', []);
var diaryServices = angular.module('webdiary.services', ['ng']);
var diaryFilters = angular.module('webdiary.filters', []);



diaryDirectives.directive('ngRightClick', function ($parse) {
    return function (scope, element, attrs) {
        var fn = $parse(attrs.ngRightClick);
        element.bind('contextmenu', function (event) {
            scope.$apply(function () {
                event.preventDefault();
                fn(scope, {$event: event});
            });
        });
    };
});

diaryFilters.filter('split', function ($log) {
    return function (input, splitChar, splitIndex) {
        // do some bounds checking here to ensure it has that index
        $log.info(input);
        if (input === undefined || input === null)
            return input;

        return input.split(splitChar)[splitIndex];
    };
});